@@ -166,13 +166,7 @@ Requires jQuery, core.js, and SelectBox.js.
                 // In horizontal mode, give the same height to the two boxes.
                 var j_from_box = $('#' + field_id + '_from');
                 var j_to_box = $('#' + field_id + '_to');
-                var resize_filters = function() { j_to_box.height($(filter_p).outerHeight() + j_from_box.outerHeight()); };
-                if (j_from_box.outerHeight() > 0) {
-                    resize_filters(); // This fieldset is already open. Resize now.
-                } else {
-                    // This fieldset is probably collapsed. Wait for its 'show' event.
-                    j_to_box.closest('fieldset').one('show.fieldset', resize_filters);
-                }
+                j_to_box.height($(filter_p).outerHeight() + j_from_box.outerHeight());
             }
 
             // Initial icon refresh
@@ -185,11 +179,11 @@ Requires jQuery, core.js, and SelectBox.js.
                 // This is much faster in WebKit browsers than the fallback.
                 field.attr('required', 'required');
                 any_selected = field.is(':valid');
-                field.removeAttr('required');
             } catch (e) {
                 // Browsers that don't support :valid (IE < 10)
                 any_selected = field.find('option:selected').length > 0;
             }
+            field.removeAttr('required');
             return any_selected;
         },
         refresh_icons: function(field_id) {
