@@ -39,7 +39,7 @@ class AccessMixin:
         return self.redirect_field_name
 
     def handle_no_permission(self):
-        if self.raise_exception:
+        if self.raise_exception or self.request.user.is_authenticated:
             raise PermissionDenied(self.get_permission_denied_message())
         return redirect_to_login(self.request.get_full_path(), self.get_login_url(), self.get_redirect_field_name())
 
@@ -67,7 +67,7 @@ class PermissionRequiredMixin(AccessMixin):
                 '{0}.get_permission_required().'.format(self.__class__.__name__)
             )
         if isinstance(self.permission_required, str):
-            perms = (self.permission_required, )
+            perms = (self.permission_required,)
         else:
             perms = self.permission_required
         return perms
