@@ -17,7 +17,7 @@ from django.contrib.gis.gdal.raster.const import (
 )
 from django.contrib.gis.gdal.srs import SpatialReference, SRSException
 from django.contrib.gis.geometry import json_regex
-from django.utils.encoding import force_bytes, force_text
+from django.utils.encoding import force_bytes, force_str
 from django.utils.functional import cached_property
 
 
@@ -31,7 +31,7 @@ class TransformPoint(list):
     def __init__(self, raster, prop):
         x = raster.geotransform[self.indices[prop][0]]
         y = raster.geotransform[self.indices[prop][1]]
-        list.__init__(self, [x, y])
+        super().__init__([x, y])
         self._raster = raster
         self._prop = prop
 
@@ -233,7 +233,7 @@ class GDALRaster(GDALRasterBase):
         Return the name of this raster. Corresponds to filename
         for file-based rasters.
         """
-        return force_text(capi.get_ds_description(self._ptr))
+        return force_str(capi.get_ds_description(self._ptr))
 
     @cached_property
     def driver(self):
@@ -381,27 +381,14 @@ class GDALRaster(GDALRasterBase):
         consult the GDAL_RESAMPLE_ALGORITHMS constant.
         """
         # Get the parameters defining the geotransform, srid, and size of the raster
-        if 'width' not in ds_input:
-            ds_input['width'] = self.width
-
-        if 'height' not in ds_input:
-            ds_input['height'] = self.height
-
-        if 'srid' not in ds_input:
-            ds_input['srid'] = self.srs.srid
-
-        if 'origin' not in ds_input:
-            ds_input['origin'] = self.origin
-
-        if 'scale' not in ds_input:
-            ds_input['scale'] = self.scale
-
-        if 'skew' not in ds_input:
-            ds_input['skew'] = self.skew
-
+        ds_input.setdefault('width', self.width)
+        ds_input.setdefault('height', self.height)
+        ds_input.setdefault('srid', self.srs.srid)
+        ds_input.setdefault('origin', self.origin)
+        ds_input.setdefault('scale', self.scale)
+        ds_input.setdefault('skew', self.skew)
         # Get the driver, name, and datatype of the target raster
-        if 'driver' not in ds_input:
-            ds_input['driver'] = self.driver.name
+        ds_input.setdefault('driver', self.driver.name)
 
         if 'name' not in ds_input:
             ds_input['name'] = self.name + '_copy.' + self.driver.name
