@@ -3,15 +3,33 @@ import datetime
 from .base import Database
 
 
-class InsertIdVar:
+class InsertVar:
     """
     A late-binding cursor variable that can be passed to Cursor.execute
     as a parameter, in order to receive the id of the row created by an
     insert statement.
     """
+    types = {
+        'AutoField': int,
+        'BigAutoField': int,
+        'SmallAutoField': int,
+        'IntegerField': int,
+        'BigIntegerField': int,
+        'SmallIntegerField': int,
+        'PositiveSmallIntegerField': int,
+        'PositiveIntegerField': int,
+        'FloatField': Database.NATIVE_FLOAT,
+        'DateTimeField': Database.TIMESTAMP,
+        'DateField': Database.Date,
+        'DecimalField': Database.NUMBER,
+    }
+
+    def __init__(self, field):
+        internal_type = getattr(field, 'target_field', field).get_internal_type()
+        self.db_type = self.types.get(internal_type, str)
 
     def bind_parameter(self, cursor):
-        param = cursor.cursor.var(Database.NUMBER)
+        param = cursor.cursor.var(self.db_type)
         cursor._insert_id_var = param
         return param
 
@@ -33,6 +51,7 @@ class Oracle_datetime(datetime.datetime):
 
 class BulkInsertMapper:
     BLOB = 'TO_BLOB(%s)'
+    CLOB = 'TO_CLOB(%s)'
     DATE = 'TO_DATE(%s)'
     INTERVAL = 'CAST(%s as INTERVAL DAY(9) TO SECOND(6))'
     NUMBER = 'TO_NUMBER(%s)'
@@ -52,5 +71,6 @@ class BulkInsertMapper:
         'PositiveIntegerField': NUMBER,
         'PositiveSmallIntegerField': NUMBER,
         'SmallIntegerField': NUMBER,
+        'TextField': CLOB,
         'TimeField': TIMESTAMP,
     }
