@@ -1,5 +1,4 @@
 import re
-import warnings
 from urllib.parse import urlparse
 
 from django.conf import settings
@@ -7,10 +6,8 @@ from django.core.exceptions import PermissionDenied
 from django.core.mail import mail_managers
 from django.http import HttpResponsePermanentRedirect
 from django.urls import is_valid_path
-from django.utils.cache import (
-    cc_delim_re, get_conditional_response, set_response_etag,
-)
-from django.utils.deprecation import MiddlewareMixin, RemovedInDjango21Warning
+from django.utils.deprecation import MiddlewareMixin
+from django.utils.http import escape_leading_slashes
 
 
 class CommonMiddleware(MiddlewareMixin):
@@ -30,11 +27,6 @@ class CommonMiddleware(MiddlewareMixin):
 
           This behavior can be customized by subclassing CommonMiddleware and
           overriding the response_redirect_class attribute.
-
-        - ETags: If the USE_ETAGS setting is set, ETags will be calculated from
-          the entire page content and Not Modified responses will be returned
-          appropriately. USE_ETAGS is deprecated in favor of
-          ConditionalGetMiddleware.
     """
 
     response_redirect_class = HttpResponsePermanentRedirect
@@ -46,9 +38,10 @@ class CommonMiddleware(MiddlewareMixin):
         """
 
         # Check for denied User-Agents
-        if 'HTTP_USER_AGENT' in request.META:
+        user_agent = request.META.get('HTTP_USER_AGENT')
+        if user_agent is not None:
             for user_agent_regex in settings.DISALLOWED_USER_AGENTS:
-                if user_agent_regex.search(request.META['HTTP_USER_AGENT']):
+                if user_agent_regex.search(user_agent):
                     raise PermissionDenied('Forbidden user agent')
 
         # Check for a redirect based on settings.PREPEND_WWW
@@ -88,6 +81,8 @@ class CommonMiddleware(MiddlewareMixin):
         POST, PUT, or PATCH.
         """
         new_path = request.get_full_path(force_append_slash=True)
+        # Prevent construction of scheme relative urls.
+        new_path = escape_leading_slashes(new_path)
         if settings.DEBUG and request.method in ('POST', 'PUT', 'PATCH'):
             raise RuntimeError(
                 "You called this URL via %(method)s, but the URL doesn't end "
@@ -103,8 +98,6 @@ class CommonMiddleware(MiddlewareMixin):
 
     def process_response(self, request, response):
         """
-        Calculate the ETag, if needed.
-
         When the status code of the response is 404, it may redirect to a path
         with an appended slash if should_redirect_with_slash() returns True.
         """
@@ -114,23 +107,6 @@ class CommonMiddleware(MiddlewareMixin):
             if self.should_redirect_with_slash(request):
                 return self.response_redirect_class(self.get_full_path_with_slash(request))
 
-        if settings.USE_ETAGS and self.needs_etag(response):
-            warnings.warn(
-                "The USE_ETAGS setting is deprecated in favor of "
-                "ConditionalGetMiddleware which sets the ETag regardless of "
-                "the setting. CommonMiddleware won't do ETag processing in "
-                "Django 2.1.",
-                RemovedInDjango21Warning
-            )
-            if not response.has_header('ETag'):
-                set_response_etag(response)
-
-            if response.has_header('ETag'):
-                return get_conditional_response(
-                    request,
-                    etag=response['ETag'],
-                    response=response,
-                )
         # Add the Content-Length header to non-streaming responses if not
         # already set.
         if not response.streaming and not response.has_header('Content-Length'):
@@ -138,11 +114,6 @@ class CommonMiddleware(MiddlewareMixin):
 
         return response
 
-    def needs_etag(self, response):
-        """Return True if an ETag header should be added to response."""
-        cache_control_headers = cc_delim_re.split(response.get('Cache-Control', ''))
-        return all(header.lower() != 'no-store' for header in cache_control_headers)
-
 
 class BrokenLinkEmailsMiddleware(MiddlewareMixin):
 
@@ -163,7 +134,8 @@ class BrokenLinkEmailsMiddleware(MiddlewareMixin):
                     ),
                     "Referrer: %s\nRequested URL: %s\nUser agent: %s\n"
                     "IP address: %s\n" % (referer, path, ua, ip),
-                    fail_silently=True)
+                    fail_silently=True,
+                )
         return response
 
     def is_internal_request(self, domain, referer):
